/*

  fatal - fatal error routines
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "fatal.hpp"

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <errno.h>

static void intro ()
{
  fprintf(stderr,"\n\n");
};

static void extro ()
{
  fprintf(stderr,"\n\n");
};

static void coredump ()
{
  fprintf(stderr,"I will now try to dump core\n");
  raise (SIGSEGV);
  fprintf(stderr,"I couldn't dump core. i will now kill myself\n");
  raise (SIGKILL);
};

static void fatalintro ()
{
  fprintf(stderr,
	  "FATAL ERROR ** FATAL ERROR ** FATAL ERROR\n"
	  "-----------------------------------------\n");
};

static void warningintro ()
{
  fprintf(stderr,"**  WARNING -- WARNING -- WARNING -- WARNING  **\n");
};

void fatal (const char *const msg)
{
  intro();
  fatalintro();
  fprintf(stderr,"%s\n", msg);
  coredump();
};

void fatal (const string &msg)
{
  fatal(msg.c_str());
};

void fatal_errno (const char *const msg)
{
  intro();
  fatalintro();
  fprintf(stderr,"%s\nerrno: %s\n", msg, strerror(errno));
  coredump();
};

void fatal_errno (const string &msg)
{
  fatal_errno(msg.c_str());
};

void warning (const char *const msg)
{
  intro();
  warningintro();
  fprintf(stderr,"%s\n", msg);
  extro();
};

void warning (const string &msg)
{
  warning(msg.c_str());
};
