#include "memchk.hpp"
#include "yyypostman.hpp"
#include "yyynetwork.hpp"
#include "fatal.hpp"
#include "yyyscheduler.hpp"
#include "yyyexceptions.hpp"
#include "yyybinbuff.hpp"
#include "yyysendables.hpp"
#include "yyysigdeferer.hpp"
#include "sendsmartptr.hpp"
#include "compsprite.hpp"
#include "ident.hpp"
#include "sendbinbuff.hpp"
#include "sendstring.hpp"
#include "visor.hpp"

#include <unistd.h>

using namespace yyybinbuff;
using namespace yyyscheduler;
using namespace yyypostman;
using namespace yyynetwork;
using namespace yyysmartptr;
using namespace yyysigdeferer;
using namespace SigC;

class serverprocess;
class peer_t;

typedef connection<scheduler,binbuff> con_t;
typedef smartptr<con_t> sptr_con_t;
typedef server<scheduler,binbuff> server_t;
typedef postman<const uint8,con_t> postman_t;
typedef smartptr<postman_t> sptr_postman_t;
typedef smartptr<peer_t> sptr_peer_t;
typedef smartptr<binbuff> sptr_binbuff;

static ident_t nextid=1;

class peer_t : public Object {
public:
  sptr_con_t con;
  sptr_postman_t post;
  serverprocess &sp;

  Signal1<void,peer_t*> killme;

  peer_t(serverprocess &sp_, sptr_con_t &c);

  ~peer_t()
  {
    cout << "destructor of peer ";
    debug();
    cout << " called.\n";
    post.kill(); // must be killed before it's connection!
  };

  void disconnected(con_t &)
  {
    debug();
    cout << " disconnected !\n";
    killme.emit(this);
  };

  void corruptreceived(postman_t &)
  {
    debug();
    cout << " postman corruptreceived, cutting connection ...\n";
    killme.emit(this);
  };

  void debug()
  {
    cout << '[' << con->getforeign_ipv4address().dump() << ']';
  };
};

class serverprocess : public Object
{
public:
  map<ident_t, sptr_compsprite> compsprite_map;
  map<ident_t, sptr_visor> visor_map;
  
  server_t myserver;
  scheduler::ionotify io;

  list<sptr_peer_t> mycons;
  sigdeferer1<peer_t*, scheduler> killpeerdefer;

  serverprocess (int argc, char *argv[])
  {
    myserver.resultavailable.connect
      (slot(*this,&serverprocess::newconnection));
    myserver.setport(10002);

    io.fire.connect(slot(*this,&serverprocess::keypressed));
    io.setfilehandle(0,scheduler::ionotify::READ);

    killpeerdefer.send.connect(slot(*this,&serverprocess::killpeer));

    cout << "Enter 'q' to quit server\n";
  };

  void newconnection(server_t &, sptr_con_t &c)
  {
    sptr_peer_t np(new peer_t(*this,c));
    np->killme.connect(slot(killpeerdefer,&sigdeferer1<peer_t*,scheduler>::receive));
    mycons.push_back(np);
  };

  void killpeer(peer_t *p)
  {
    list<sptr_peer_t>::iterator i;
    for (i=mycons.begin();i!=mycons.end();++i) {
      if (&(**i) == p) {
	mycons.erase(i);
	return;
      };
    };
    fatal("serverprocess::killpeer didn't find it's peer");
  };

  void keypressed(scheduler::ionotify &)
  {
    char c;
    read(0,&c,1);
    if (c=='q') {
      cout << "shutting down...\n";
      scheduler::gsched->setterminated(true);
    };
  };

  /*  void recv_text(const MSG_cc &m)
  {
    if ( ! ((m.cl.ascii>=32)&&(m.cl.ascii<=127)) ) {
      cout << "recved illegal ascii code\n";
      return;
    };
    cout << "[" << int(m.x) << ',' << int(m.y) << ',' << m.cl.ascii << ',' << int(m.cl.colnum) << "]\n";
    if ((m.x<0) || (m.y<0) || (m.x>=area.xsize()) || (m.y>=area.ysize())) {
      cout << "that was illegal coordinates, ignoring !\n";
      return;
    };
    if ( ! ((m.cl.colnum>=1)&&(m.cl.colnum<=22)) ) {
      cout << "that was illegal color, ignoring !\n";
      return;
    };
    area(m.x,m.y)=m.cl;
    list<sptr_peer_t>::iterator i;
    for (i=mycons.begin();i!=mycons.end();++i)
      (*i)->post->sender1<const MSG_cc &> ('s') (m);
      };*/

  /*  void recv_new_graphic(const sptr_graphic_t &g)
  {
    g->id=gidpool->getfreeid();
    (*ggraphmap)[g->id]=g;
    dumpgraphmap();
    };*/

  void binbuff_new (sptr_binbuff &b, const string &s)
  {
    cout << "binbuff_new\n";
    
    ident_t myid = nextid++;
    sptr_compsprite c(new compsprite());
    c->filebuf=b;
    c->name=s;
    compsprite_map[myid]=c;
    list<sptr_peer_t>::iterator i;
    for (i=mycons.begin();i!=mycons.end();++i)
      (*i)->post->sender2<sptr_compsprite &, ident_t> ('b') (c, myid);
  };

  void visor_new (sptr_visor &v)
  {
    cout << "visor_new\n";
    
    ident_t myid = nextid++;
    visor_map[myid]=v;
    list<sptr_peer_t>::iterator i;
    for (i=mycons.begin();i!=mycons.end();++i)
      (*i)->post->sender2<sptr_visor &, ident_t> ('d') (v, myid);
  };

  void visor_adjust (sptr_visor &v, ident_t id)
  {
    cout << "visor_adjust\n";
    
    if (visor_map.find(id)==visor_map.end()) return;
    visor_map[id]=v;
    list<sptr_peer_t>::iterator i;
    for (i=mycons.begin();i!=mycons.end();++i)
      (*i)->post->sender2<sptr_visor &, ident_t> ('e') (v, id);
  };
  

  /*  void dumpgraphmap()
  {
    for (graphicsmap_t::const_iterator i=ggraphmap->begin();i!=ggraphmap->end();++i) {
      cout << "name ["<<i->second->name<<"]  id ["<<i->second->id<<"]\n";
    };
    };*/
};





peer_t::peer_t(serverprocess &sp_, sptr_con_t &c)
  : sp(sp_)
{
  con=c;
  
  con->sig_disconnected.connect
    (slot(*this,&peer_t::disconnected));
  
  post=sptr_postman_t(new postman_t(&(*con)));
  
  post->corruptreceived.connect
    (slot(*this,&peer_t::corruptreceived));
  
  /*  post->receiver1_open<const MSG_cc &> ('x')
    .connect(slot(sp,&serverprocess::recv_text));

  post->sender1_open<const MSG_cc &> ('s');

  post->sender1_open<const tempm_cell &> ('a');
  post->sender1<const tempm_cell &> ('a') (sp.area);*/

  /*  post->receiver1_open<const sptr_graphic_t&> (0)
      .connect(slot(sp,&serverprocess::recv_new_graphic));*/

  post->receiver2_open<sptr_binbuff &, const string &> ('a')
    .connect(slot(sp,&serverprocess::binbuff_new));

  post->sender2_open<sptr_compsprite &, ident_t> ('b');

  post->receiver1_open<sptr_visor &> ('c')
    .connect(slot(sp,&serverprocess::visor_new));

  post->sender2_open<sptr_visor &, ident_t> ('d');

  post->sender2_open<sptr_visor &, ident_t> ('e');
    
  post->receiver2_open<sptr_visor &, ident_t> ('e')
    .connect(slot(sp,&serverprocess::visor_adjust));

  {
    map<ident_t,sptr_compsprite>::iterator i;
    for (i=sp.compsprite_map.begin();i!=sp.compsprite_map.end();++i)
      post->sender2<sptr_compsprite &, ident_t> ('b') (i->second, i->first);
  };
  {
    map<ident_t,sptr_visor>::iterator i;
    for (i=sp.visor_map.begin();i!=sp.visor_map.end();++i)
      post->sender2<sptr_visor &, ident_t> ('d') (i->second, i->first);
  };
  
  cout << "new connection from ";
  debug();
  cout << ".\n";
};





int main(int argc, char *argv[])
{
  memchk::resetsituation();
  {
    yyyexceptions::except_init();
    try {
      scheduler s;
      scheduler::gsched = &s;
      serverprocess serv(argc,argv);
      s.run();
    } catch (yyyexceptions::except_base &x) {
      fatal(x.humandump());
    } catch (const char *const x) {
      fatal(x);
    } catch (...) {
      fatal("unspecified (...) exception caught.");
    };
  };
  if (!memchk::everythingzero()) memchk::printstatistics(cout);
  
  return 0;
};
