/*

  yyybinbuff - a implementation of the binbuffer concept
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "yyybinbuff.hpp"

using namespace yyybinbuff;

binbuff::binbuff ()
  : buf(new uint8 [0]), offs(0), len(0), cap(0)
{
};

binbuff::~binbuff ()
{
  delete [] buf;
};

void binbuff::resize (const index s)
{
  if (s<0) throw rangeerror();
  if (s>=size())
    add_end(s-size());
  else
    remove_end(size()-s);
};

void binbuff::erase (const index first, const index last)
{
  if (first==0) {
    remove_begin (last);
  } else throw rangeerror();
};

void binbuff::add_end (const index amount)
{
  if (amount < 0) throw rangeerror();
  index newlen = len + amount;
  
  if (offs + newlen > cap) {
    cap = newlen * 2;
    uint8 *newbuf = new uint8 [cap];
    memcpy (newbuf, buf + offs, len);
    delete [] buf;
    buf = newbuf;
    offs = 0;
  };
  len = newlen;
};

void binbuff::remove_end (const index amount)
{
  if (amount < 0) throw rangeerror();
  collapse (offs, len - amount);
};

void binbuff::remove_begin (const index amount)
{
  if (amount < 0) throw rangeerror();
  collapse (offs + amount, len - amount);
};

void binbuff::collapse (index start, index length)
{
  if (length < 0) throw rangeerror();
  len = length;
  
  if (len < cap / 4) {
    cap = len * 2;
    uint8 *newbuf = new uint8 [cap];
    memcpy (newbuf, buf + start, len);
    delete [] buf;
    buf = newbuf;
    offs = 0;
  } else {
    offs = start;
  };
};
